#!/bin/sh
#set -x

print_Usage() {
		echo ""
		echo "-------- QLogic Corporation -- Firmaware Dump Utility --------"
		echo ""
		echo "./qlcnic_dump.sh -l ethX|ALL"
		echo "Display firmware dump information on a \"single\" or \"all\" qlcnic adapter"
		echo ""
		echo "./qlcnic_dump.sh -s ethX|ALL"
		echo "Save firmware dump from a \"single\" or \"all\" qlcnic adapter"
		echo "firmware dump will be saved in directory \"/opt/QLogic_Corporation/FW_Dumps\""
		echo "Saved file will be in \"qlcnic_fw_dump_ethX_date_time.bin\" format"
		echo ""
		echo "./qlcnic_dump.sh -f ethX|ALL"
		echo "Force firmware dump on a \"single\" or \"all\" qlcnic adapter"
		echo ""
		echo "./qlcnic_dump.sh -c ethX|ALL 1|0"
		echo "Enable(1)/Disable(0) firmware dump on a \"single\" or \"all\" qlcnic adapter"
		echo ""
		echo "./qlcnic_dump.sh -m ethX|ALL <Mask-Value>"
		echo "Change the current firmware dump capture mask on a \"single\" or \"all\" qlcnic adapter"
		echo "Available firmware dump capture mask values are between 0x03 (Min) and 0xff (Max)"
		echo "Default firmware dump capture mask value is 0x1f"
		echo ""
		exit 1
}


err() {
		echo "$@" >&2
		if [ -x /bin/logger ]; then
			/bin/logger -t "${0##*/}[$$]" "$@"
		fi
}

# This Function will find All the interfaces on a single Adapter
Find_interfaces_single() {

		k=0
		m=0
		Drvr=`ethtool -i $ETH | grep qlcnic`
		if [ "$Drvr" == "" ] ; then
			echo "qlcnic: Please pass valid interface"
			echo "qlcnic: Operation aborted"
			print_Usage
		fi
		Flag=`find /sys/devices -name "$ETH" | grep "/net/$ETH"`

		if [ "$Flag" == "" ] ; then
			eth_dir="find /sys/devices -name "net:$ETH""
			Bus=`$eth_dir | awk -F/ '{ for (i = NF; i > 0; --i) print $i }' | head -2 | grep "0000" | cut -d'.' -f1 | cut -d':' -f2,3`
		else
			eth_dir="find /sys/devices -name "$ETH""
			Bus=`$eth_dir | awk -F/ '{ for (i = NF; i > 0; --i) print $i }' | head -3 | grep "0000" | cut -d'.' -f1 | cut -d':' -f2,3`
		fi

		#Check the Bus info with Ethtool if no valid Bus is Determined Corresponding to an interface
		if [ "$Bus" == "" ] ; then
			Bus=`ethtool -i $ETH | grep bus | cut -d' ' -f 2 | cut -d':' -f2,3 | cut -d'.' -f1`

			if [ "$Bus" == "" ] ; then
				echo "qlcnic: Please check the argument $ETH passed"
				echo "qlcnic: Operation aborted"
				print_Usage
			fi
		fi

		for i in `/sbin/lspci -d1077:| grep Ethernet| cut -d' ' -f 1 | grep "$Bus"`
		do
			ls /sys/bus/pci/devices/0000\:"$i"/net/ &>/dev/null
			brr[m]="0000:"$i""
			if [ $? -eq 0 ] ; then
				arr[k]=`ls /sys/bus/pci/devices/0000\:"$i"/net/`
			else
				arr[k]=`ls /sys/bus/pci/devices/0000\:"$i"/ | grep net | cut -d':' -f 2`
			fi
			k=$((k+1))
			m=$((m+1))
		done
}

# This Function will find All the interfaces on All Adapters Present in the System
Find_interfaces_all() {

		k=0
		m=0
		if [ "$option" == "-f" -a "$ETH" == "ALL" ] ; then
			for i in `/sbin/lspci -d1077:| grep Ethernet| cut -d' ' -f 1 | cut -d'.' -f1 | uniq`
			do
				ls /sys/bus/pci/devices/0000\:"$i".0/net/ &>/dev/null
				if [ $? -eq 0 ] ; then
					arr[k]=`ls /sys/bus/pci/devices/0000\:"$i".0/net/`
				else
					arr[k]=`ls /sys/bus/pci/devices/0000\:"$i".0/ | grep net | cut -d':' -f 2`
				fi
				k=$((k+1))
			done
		else
			for i in `/sbin/lspci -d1077:| grep Ethernet| cut -d' ' -f 1`
			do
				ls /sys/bus/pci/devices/0000\:"$i"/net/ &>/dev/null
				brr[m]="0000:"$i""
				if [ $? -eq 0 ] ; then
					arr[k]=`ls /sys/bus/pci/devices/0000\:"$i"/net/`
				else
					arr[k]=`ls /sys/bus/pci/devices/0000\:"$i"/ | grep net | cut -d':' -f 2`
				fi
				k=$((k+1))
				m=$((m+1))
			done
		fi
}

Save_dump() {
		echo ""
		mkdir -p $DFILE_PATH &> /dev/null

		if [ "$ETH" == "ALL" ] ; then
			Find_interfaces_all
		else
			Find_interfaces_single
		fi

		Check_driver
		index=0
		while [ $index -lt ${#arr[@]} ]
		do
			QFWD=${SYSFS}/class/net/${arr[$index]}/device/fw_dump
			QFWDS=${SYSFS}/class/net/${arr[$index]}/device/fwdump_size
			DFILE=${DFILE_PATH}/qlcnic_fw_dump_"${arr[$index]}"_`eval date +%Y%m%d_%H%M%S`.bin

			if ! test -f ${QFWD} ; then
				err "qlcnic ${arr[$index]}: doesn't support firmware dump capability"
				index=$(( $index + 1 ))
			else
				size=`cat ${QFWDS}`
				if [ "$size" -ne 0 ] ; then
					(( remainder = size%4096 ))
					(( loops = size/4096 ))
					if [ $remainder != "0" ] ; then
						(( loops++ ))
					fi
					for (( i = 0; i < $loops ; i++ ))
					do
						cat ${QFWD} >> ${DFILE}
					done

					if ! test -s "${DFILE}" ; then
						rm ${DFILE}
						index=$(( $index + 1 ))
					else
						err "qlcnic ${arr[$index]}: Firmware dump saved to file "${DFILE}" "
						index=$(( $index + 1 ))
					fi
				else
					echo "qlcnic ${arr[$index]}: Firmware dump is not available"
					index=$(( $index + 1 ))
				fi
			fi
		done
		echo ""
}

Force_dump() {
		if [ "$NOA" -ne 2 ] ; then
			echo "qlcnic: Forcing the firmware dump failed"
			echo "qlcnic: Please provide valid number of arguments"
			print_Usage
		else
			if [ "$ETH" == "ALL" ] ; then
				Find_interfaces_all
				Check_driver

				index=0
				while [ $index -lt ${#arr[@]} ]
				do
					ls ${SYSFS}/class/net/${arr[$index]}/device/fw_dump &>/dev/null
					if [ $? -ne 0 ] ; then
						echo "qlcnic ${arr[$index]}: Does not support firmware dump capability"
						index=$(( $index + 1 ))
					else
						size=`cat /sys/class/net/${arr[$index]}/device/fwdump_size`
						if [ "$size" -ne 0 ] ; then
							echo "qlcnic ${arr[$index]}: Please save the firmware dump first"
							echo "qlcnic ${arr[$index]}: Force firmware dump cannot be performed"
							echo ""
							index=$(( $index + 1 ))
						else
							DS=`cat /sys/class/net/${arr[$index]}/device/fwdump_state`
							if [ $DS -ne 1 ] ; then
								echo ""
								echo "qlcnic ${arr[$index]}: Firmware dump is not enabled"
								echo "qlcnic ${arr[$index]}: Force firmware dump cannot be performed"
								echo ""
								index=$(( $index + 1 ))
							else
								ethtool -i ${arr[$index]} &>/dev/null
								if [ $? -ne 0 ] ; then
									echo ""
									echo "qlcnic ${arr[$index]}: is not available"
									echo "qlcnic ${arr[$index]}: Force Dump aborted"
									echo ""
									index=$(( $index + 1 ))
								else
									echo 0xdeadfeed > /sys/class/net/${arr[$index]}/device/fw_dump
									size=`cat /sys/class/net/${arr[$index]}/device/fwdump_size`
									while [ "$size" -eq 0 ]
									do
										size=`cat /sys/class/net/${arr[$index]}/device/fwdump_size`
										sleep 2
									done
									echo "qlcnic ${arr[$index]}: Force firmware dump performed"
									index=$(( $index + 1 ))
								fi
							fi
						fi
					fi
				done
				echo ""
			else
				ethtool -i $ETH &>/dev/null
				if [ "$?" -ne 0 ] ; then
					echo ""
					echo "qlcnic $ETH: is not available"
					echo "qlcnic $ETH: Force Dump aborted"
					echo ""
					print_Usage
				fi

				ls ${SYSFS}/class/net/$ETH/device/fw_dump &>/dev/null
				if [ $? -ne 0 ] ; then
					echo "qlcnic $ETH: does not support firmware dump capability"
					exit 0
				else
					size=`cat /sys/class/net/$ETH/device/fwdump_size`
					if [ "$size" -ne 0 ] ; then
						echo "qlcnic $ETH: Please save the dump first"
						echo "qlcnic $ETH: Force firmware dump cannot be performed"
						echo ""
						exit 0
					else
						DS=`cat /sys/class/net/$ETH/device/fwdump_state`
						if [ $DS -ne 1 ] ; then
							echo "qlcnic $ETH: Dump is not enabled"
							echo "qlcnic $ETH: Force firmware dump cannot be performed"
							echo ""
						else
							echo 0xdeadfeed > /sys/class/net/$ETH/device/fw_dump
							size=`cat /sys/class/net/$ETH/device/fwdump_size`
							while [ "$size" -eq 0 ]
							do
								size=`cat /sys/class/net/$ETH/device/fwdump_size`
								sleep 2
							done
							echo "qlcnic $ETH: Force firmware dump performed"
							exit 0
						fi
					fi
				fi
			fi
		fi
}


Check_driver() {

		count=0
		index=0
		while [ $index -lt ${#arr[@]} ]
		do
			if [ "${arr[$index]}" == "" ] ; then
				index=$(( $index + 1 ))
			else
				count=$(( $count + 1 ))
				index=$(( $index + 1 ))
			fi
		done

		if [ $count -eq 0 ] ; then
			echo "qlcnic Driver is not loaded"
			echo "Operation Aborted"
			exit 1
		fi
}


Disable_dump() {
		echo "** qlcnic: Disabling the firmware dump **"
		echo ""
		if [ "$NOA" -ne 3 ] ; then
			echo "qlcnic: Disabling the firmware dump failed"
			echo "qlcnic: Please provide valid number of arguments"
			print_Usage
		else
			if [ "$ETH" == "ALL" ] ; then
				Find_interfaces_all
			else
				Find_interfaces_single
			fi
			Check_driver
			index=0
			while [ $index -lt ${#arr[@]} ]
			do
				ls ${SYSFS}/class/net/${arr[$index]}/device/fw_dump &>/dev/null
				if [ $? -ne 0 ] ; then
					echo "qlcnic ${arr[$index]}: Does not support firmware dump capability"
					index=$(( $index + 1 ))
				else
					echo 0xbadfeed > ${SYSFS}/class/net/${arr[$index]}/device/fw_dump
					echo "qlcnic ${arr[$index]}: Firmware dump is disabled"
					index=$(( $index + 1 ))
				fi
			done
			echo ""
		fi
}

Enable_dump() {
		echo "** qlcnic: Enabling the firmware dump **"
		echo ""
		if [ "$NOA" -ne 3 ] ; then
			echo "qlcnic: Enabling the firmware dump failed"
			echo "qlcnic: Please provide valid number of arguments"
			print_Usage
		else
			if [ "$ETH" == "ALL" ] ; then
				Find_interfaces_all
			else
				Find_interfaces_single
			fi
			Check_driver
			index=0
			while [ $index -lt ${#arr[@]} ]
			do
				ls ${SYSFS}/class/net/${arr[$index]}/device/fw_dump &>/dev/null
				if [ $? -ne 0 ] ; then
					echo "qlcnic ${arr[$index]}: Does not support firmware dump capability"
					index=$(( $index + 1 ))
				else
					echo 0xaddfeed > ${SYSFS}/class/net/${arr[$index]}/device/fw_dump
					echo "qlcnic ${arr[$index]}: Firmware dump is enabled"
					index=$(( $index + 1 ))
				fi
			done
			echo ""
		fi
}


Change_mask() {
		if [ "$NOA" -ne 3 ] ; then
			echo "qlcnic: Cannot change the firmware dump capture mask "
			echo "qlcnic: Please provide valid number of arguments"
			print_Usage
		else
			if [ "$ETH" == "ALL" ] ; then
				Find_interfaces_all
			else
				Find_interfaces_single
                        fi

			Check_driver
			index=0
			while [ $index -lt ${#arr[@]} ]
			do
				ls ${SYSFS}/class/net/${arr[$index]}/device/fwdump_level &>/dev/null
				if [ $? -ne 0 ] ; then
					echo "qlcnic ${arr[$index]}: Does not support firmware dump capability"
					index=$(( $index + 1 ))
				else
					mask=`printf "%d\n" ${MASK}`
					if [ $mask -gt 255 -o $mask -lt 3 ] ; then
						echo "qlcnic: Invalid capture mask \"${MASK}\" "
						echo "Operation aborted"
						print_Usage
					else
						echo ${MASK} > "${SYSFS}/class/net/${arr[$index]}/device/fwdump_level"
						echo "qlcnic ${arr[$index]}: Firmware dump capture mask changed to: \"${MASK}\""
						index=$(( $index + 1 ))
					fi
				fi
			done
		fi
}

Display() {
		if [ "$NOA" -ne 2 ] ; then
			echo "qlcnic: Cannot display firmware dump information"
			echo "qlcnic: Please provide valid number of arguments"
			print_Usage
		else
			if [ "$ETH" == "ALL" ] ; then
				Find_interfaces_all
			else
				Find_interfaces_single
			fi

			Check_driver
			index1=0
			index=0
			while [ $index -lt ${#arr[@]} ]
			do
				ls ${SYSFS}/class/net/${arr[$index]}/device/fw_dump &>/dev/null
				if [ $? -ne 0 ] ; then
					echo "qlcnic ${arr[$index]}: Does not support firmware dump capability"
					index=$(( $index + 1 ))
					index1=$(( $index1 + 1 ))
				else
					PROP1=`cat ${SYSFS}/class/net/${arr[$index]}/device/fwdump_level`
					PROP1=`printf "%x\n" $PROP1`
					PROP2=`cat ${SYSFS}/class/net/${arr[$index]}/device/fwdump_size`
					PROP3=`cat ${SYSFS}/class/net/${arr[$index]}/device/fwdump_state`
					size=`cat ${SYSFS}/class/net/${arr[$index]}/device/fwdump_size`
					if [ $PROP2 -ne 0 ] ; then
						flag="Yes"
					else
						flag="No"
					fi

					if [ $PROP3 -ne 0 ] ; then
						flag1="Yes"
					else
						flag1="No"
					fi
					echo ""
					echo "qlcnic: ${arr[$index]}: ${brr[$index1]}"
					echo "Firmware dump available		:$flag"
					echo "Firmware dump size		:$size bytes"
					echo "Firmware dump enabled		:$flag1"
					echo "Firmware dump capture mask	:0x$PROP1"
					echo ""
					index=$(( $index + 1 ))
					index1=$(( $index1 + 1 ))
				fi
			done
		fi
}

option=$1
ETH=$2
MASK=$3
SYSFS=/sys
DFILE_PATH=/opt/QLogic_Corporation/FW_Dumps
NOA=$#
QFWD1=${SYSFS}/class/net/$ETH/device/fwdump_level

# Declare An Array in which All the interface Names will be stored
declare -a arr
declare -a brr

if [ "$#" == 0 ] ; then
	print_Usage
fi

case "$option" in
-h | --help)
	print_Usage
	;;
-l)
	echo ""
	echo "** qlcnic: Displaying firmware dump information"
	echo ""
	Display
	exit 0
	;;
-s)
	echo ""
	echo "** qlcnic: Saving the firmware dump **"
	echo ""
	Save_dump
	exit 0
	;;
-f)
	echo ""
	echo "** qlcnic: Forcing the firmware dump **"
	echo ""
	Force_dump
	exit 0
	;;
-c)
	if [ "$3" == "0" -o "$3" == "1" ] ; then
		if [ $3 -ne 0 ] ; then
			echo ""
			Enable_dump
		else
			echo ""
			Disable_dump
		fi
	else
		echo ""
		echo "qlcnic: Please check the argument $3"
		print_Usage
	fi
	exit 0
	;;
-m)
	echo ""
	echo "** qlcnic: Changing the firmware dump capture mask value **"
	echo ""
	Change_mask
	exit 0
	;;
*)
	print_Usage
	;;
esac

